@@ -4,12 +4,12 @@
  * This file is part of ePiX, a C++ library for creating high-quality 
  * figures in LaTeX 
  *
- * Version 1.1.18
- * Last Change: September 18, 2007
+ * Version 1.2.5
+ * Last Change: May 11, 2008
  */
 
 /* 
- * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007
+ * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
  * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
  * Department of Mathematics and Computer Science
  * College of the Holy Cross
@@ -42,6 +42,7 @@
 #include "pairs.h"
 #include "frac.h"
 #include "functions.h"
+#include "utils.h" // for truncate
 
 #include "length.h"
 #include "picture.h"
@@ -330,8 +331,14 @@ namespace ePiX {
       for (unsigned int i=0; i <= m_major_segs; ++i)
 	{
 	  const P loc(m_tail + i*major_step);
-	  const double tmp_lbl(m_coord(loc));
-	  if (m_omitted.find(tmp_lbl) != m_omitted.end()) // omit marked labels
+	  const double tmp_lbl(truncate(m_coord(loc))); // round to 0 if nec.
+
+	  std::set<double>::const_iterator mp(m_omitted.begin());
+	  while (mp != m_omitted.end() && 
+		 EPIX_EPSILON < (fabs(tmp_lbl - *mp))) // keep label?
+	    ++mp;
+
+	  if (mp != m_omitted.end())
 	    continue;
 
 	  // else
