@@ -1,15 +1,15 @@
 /* 
  * geometry.cc -- spherical and hyperbolic geometry
  *
- * This file is part of ePiX, a preprocessor for creating high-quality 
- * line figures in LaTeX 
+ * This file is part of ePiX, a C++ library for creating high-quality 
+ * figures in LaTeX 
  *
- * Version 1.0.20
- * Last Change: October 29, 2006
+ * Version 1.1.21
+ * Last Change: September 22, 2007
  */
 
 /* 
- * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
+ * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007
  * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
  * Department of Mathematics and Computer Science
  * College of the Holy Cross
@@ -35,27 +35,73 @@
 #include <cmath>
 #include <vector>
 
-#include "globals.h"
+#include "constants.h"
 #include "errors.h"
 #include "functions.h"
 
 #include "triples.h"
+#include "frame.h"
+
+#include "state.h"
+
+#include "camera.h"
 
 #include "path.h"
 
 #include "sphere.h"
-
 #include "curves.h"
 
 #include "geometry.h"
 
 namespace ePiX {
 
+  //// Static utility functions ////
+
   // Flag for type of projection to the sphere
   enum sphere_proj_type {RADIAL, STEREO_N, STEREO_S};
 
+  // point constructor in geographic coords on specified sphere/frame
+  P sphere_posn(double lat, double lngtd, const Sphere& S, const frame& coords)
+  {
+    double rad(S.radius());
+
+    return S.center() + 
+      rad*Cos(lat)*(Cos(lngtd)*coords.sea() + Sin(lngtd)*coords.sky()) +
+      rad*Sin(lat)*coords.eye();
+  }
+
+  // latitudes and longitudes
+  void draw_latitude(double lat, double lngtd_min, double lngtd_max,
+		     bool front, const Sphere& S, const frame& coords)
+  {
+    P center(S.center() + (S.radius()*Sin(lat)*coords.eye()));
+    double radius(S.radius()*Cos(lat));
+
+    path temp(center, radius*coords.sea(), radius*coords.sky(),
+	      lngtd_min, lngtd_max);
+
+    temp.clip_to(S, cam().viewpt(), front);
+    temp.draw();
+  }
+
+  // draw portion of longitude line
+  void draw_longitude(double lngtd, double lat_min, double lat_max,
+		      bool front, const Sphere& S, const frame& coords)
+  {
+    P center(S.center());
+    double radius(S.radius());
+
+    path temp(center,
+	      radius*(Cos(lngtd)*coords.sea()+Sin(lngtd)*coords.sky() ),
+	      radius*coords.eye(), lat_min, lat_max);
+
+    temp.clip_to(S, cam().viewpt(), front);
+    temp.draw();
+  }
+
+
   // Spherical geometry
-  P proj_to_sphere(const P& arg, const sphere& S, sphere_proj_type TYPE)
+  P proj_to_sphere(const P& arg, const Sphere& S, sphere_proj_type TYPE)
   {
     P O(S.center());
     double rad(S.radius());
@@ -83,33 +129,35 @@ namespace ePiX {
   }
 
   void draw_sphereplot(double f1(double),double f2(double),double f3(double), 
-		       double t_min, double t_max, int num_pts, bool hidden, 
-		       sphere_proj_type TYPE, const sphere& S)
+		       double t_min, double t_max, int num_pts, bool front, 
+		       sphere_proj_type TYPE, const Sphere& S)
   {
-    std::vector<vertex> data(num_pts+1);
+    std::vector<P> data(num_pts+1);
     double t(t_min);
     const double dt((t_max - t_min)/num_pts);
 
     for (int i=0; i <= num_pts; ++i, t += dt)
-      data.at(i) = vertex(proj_to_sphere(P(f1(t), f2(t), f3(t)), S, TYPE));
+      data.at(i) = proj_to_sphere(P(f1(t), f2(t), f3(t)), S, TYPE);
 
-  path temp(data, false);
-  temp.draw(S, hidden);
+  path temp(data, false, false);
+  temp.clip_to(S, cam().viewpt(), front);
+  temp.draw();
   } // end of draw_sphereplot
 
   void draw_sphereplot(P Phi(double), double t_min, double t_max, 
-		       int num_pts, bool hidden, sphere_proj_type TYPE,
-		       const sphere& S)
+		       int num_pts, bool front, sphere_proj_type TYPE,
+		       const Sphere& S)
   {
-    std::vector<vertex> data(num_pts+1);
+    std::vector<P> data(num_pts+1);
     double t(t_min);
     const double dt((t_max - t_min)/num_pts);
 
     for (int i=0; i <= num_pts; ++i, t += dt)
-      data.at(i) = vertex(proj_to_sphere(Phi(t), S, TYPE));
+      data.at(i) = proj_to_sphere(Phi(t), S, TYPE);
 
-  path temp(data, false);
-  temp.draw(S, hidden);
+  path temp(data, false, false);
+  temp.clip_to(S, cam().viewpt(), front);
+  temp.draw();
   } // end of draw_sphereplot
 
 
@@ -133,7 +181,7 @@ namespace ePiX {
 
   // assumes seg contains the actual (scaled, translated) endpoints
   void draw_sphere_arc(const seg& sgmt, double t_min, double t_max, 
-		       bool front, sphere_proj_type TYPE, const sphere& S)
+		       bool front, sphere_proj_type TYPE, const Sphere& S)
   {
     P tail(sgmt(t_min)), head(sgmt(t_max));
     double cos_theta(((head-S.center())|(tail-S.center()))/pow(S.radius(),2));
@@ -147,11 +195,11 @@ namespace ePiX {
       }
     // else
 
-    int num_pts((int) ceil(EPIX_NUM_PTS*Acos(cos_theta)/epix::full_turn()));
+    int num_pts((int) ceil(EPIX_NUM_PTS*Acos(cos_theta)/full_turn()));
     if (num_pts < 2)
       num_pts=2;
 
-    std::vector<vertex> data(num_pts+1);
+    std::vector<P> data(num_pts+1);
     double t(t_min);
     const double dt((t_max - t_min)/num_pts);
 
@@ -161,22 +209,96 @@ namespace ePiX {
     for (int i=0; i <= num_pts; ++i, t += dt)
       {
 	P loc(sgmt(t) - O); // location relative to O
-	data.at(i) = vertex(S.center() + rad*recip(norm(loc))*loc);
+	data.at(i) = S.center() + rad*recip(norm(loc))*loc;
       }
 
-    path temp(data, false);
-    temp.draw(S, front);
-  } // end of draw_sphereplot
+    path temp(data, false, false);
+    temp.clip_to(S, cam().viewpt(), front);
+    temp.draw();
+  } // end of draw_sphere_arc
 
+
+  //// "Geography" and spherical plotting ////
+  void latitude(double lat, double lngtd_min, double lngtd_max,
+		const Sphere& S, const frame& coords)
+  {
+    draw_latitude(lat, lngtd_min, lngtd_max, true, S, coords);
+  }
+
+
+  void longitude(double lngtd, double lat_min, double lat_max,
+		 const Sphere& S, const frame& coords)
+  {
+    draw_longitude(lngtd, lat_min, lat_max, true, S, coords);
+  }
+
+  void back_latitude(double lat, double lngtd_min, double lngtd_max,
+		     const Sphere& S, const frame& coords)
+  {
+    draw_latitude(lat, lngtd_min, lngtd_max, false, S, coords);
+  }
+
+  void back_longitude(double lngtd, double lat_min, double lat_max,
+		      const Sphere& S, const frame& coords)
+  {
+    draw_longitude(lngtd, lat_min, lat_max, false, S, coords);
+  }
+
+
+  // spherical plotting
+  void frontplot_N(double f1(double), double f2(double),
+		   double t_min, double t_max, int num_pts, 
+		   const Sphere& S)
+  {
+    draw_sphereplot(f1, f2, zero, t_min, t_max, num_pts, true, STEREO_N, S);
+  }
+
+  void backplot_N(double f1(double), double f2(double),
+		  double t_min, double t_max, int num_pts,
+		  const Sphere& S)
+  {
+    draw_sphereplot(f1, f2, zero, t_min, t_max, num_pts, false, STEREO_N, S);
+  }
+
+  void frontplot_S(double f1(double), double f2(double),
+		   double t_min, double t_max, int num_pts, 
+		   const Sphere& S)
+  {
+    draw_sphereplot(f1, f2, zero, t_min, t_max, num_pts, true, STEREO_S, S);
+  }
+
+  void backplot_S(double f1(double), double f2(double),
+		  double t_min, double t_max, int num_pts, 
+		  const Sphere& S)
+  {
+    draw_sphereplot(f1, f2, zero, t_min, t_max, num_pts, false, STEREO_S, S);
+  }
+
+
+  // Radial projection from center
+  void frontplot_R(P phi(double), double t_min, double t_max, 
+		   int num_pts, const Sphere& S)
+  {
+    draw_sphereplot(phi, t_min, t_max, num_pts, true, RADIAL, S);
+  }
+
+  void backplot_R(P phi(double), double t_min, double t_max, 
+		  int num_pts, const Sphere& S)
+  {
+    draw_sphereplot(phi, t_min, t_max, num_pts, false, RADIAL, S);
+  }
+
+
+  //// Spherical geometry -- arcs and polyhedra ////
   // arc of great circle between non-antipodal points
-  void arc(const P& tail, const P& head, const bool front, const sphere& S)
+  void arc(const P& tail, const P& head, const bool front, const Sphere& S)
   {
     draw_sphere_arc(seg(tail, head), 0, 1, front, RADIAL, S);
   }
 
 
   // user-space functions
-  void front_arc(const P& p1, const P& p2, const sphere& S)
+  void front_arc(const P& p1, const P& p2, const Sphere& S)
   {
     const P ctr(S.center());
     const double rad(S.radius());
@@ -184,7 +306,7 @@ namespace ePiX {
     arc(ctr+(rad/norm(p1))*p1, ctr+(rad/norm(p2))*p2, true, S);
   }
 
-  void  back_arc(const P& p1, const P& p2, const sphere& S)
+  void  back_arc(const P& p1, const P& p2, const Sphere& S)
   {
     const P ctr(S.center());
     const double rad(S.radius());
@@ -193,19 +315,19 @@ namespace ePiX {
   }
 
   // join p1 to -p1 through p2
-  void front_arc2(const P& p1, const P& p2, const sphere& S)
+  void front_arc2(const P& p1, const P& p2, const Sphere& S)
   {
     front_arc(p1,  p2, S);
     front_arc(p2, -p1, S);
   }
 
-  void  back_arc2(const P& p1, const P& p2, const sphere& S)
+  void  back_arc2(const P& p1, const P& p2, const Sphere& S)
   {
     back_arc(p1,  p2, S);
     back_arc(p2, -p1, S);
   }
 
-  void front_line(const P& p1, const P& p2, const sphere& S)
+  void front_line(const P& p1, const P& p2, const Sphere& S)
   {
     front_arc( p1,  p2, S);
     front_arc( p2, -p1, S);
@@ -214,7 +336,7 @@ namespace ePiX {
   }
 
 
-  void back_line(const P& p1, const P& p2, const sphere& S)
+  void back_line(const P& p1, const P& p2, const Sphere& S)
   {
     back_arc( p1,  p2, S);
     back_arc( p2, -p1, S);
@@ -223,14 +345,14 @@ namespace ePiX {
   }
 
 
-  void front_triangle(const P& p1, const P& p2, const P& p3, const sphere& S)
+  void front_triangle(const P& p1, const P& p2, const P& p3, const Sphere& S)
   {
     front_arc(p1, p2, S);
     front_arc(p2, p3, S);
     front_arc(p3, p1, S);
   }
 
-  void  back_triangle(const P& p1, const P& p2, const P& p3, const sphere& S)
+  void  back_triangle(const P& p1, const P& p2, const P& p3, const Sphere& S)
   {
     back_arc(p1, p2, S);
     back_arc(p2, p3, S);
@@ -238,7 +360,7 @@ namespace ePiX {
   }
 
   // local to this file
-  void front_dual(const P& p1, const P& p2, const P& p3, const sphere& S)
+  void front_dual(const P& p1, const P& p2, const P& p3, const Sphere& S)
   {
     const P ctr(0.3333*(p1+p2+p3));
 
@@ -247,7 +369,7 @@ namespace ePiX {
     front_arc(ctr, 0.5*(p3+p1), S);
   }
 
-  void  back_dual(const P& p1, const P& p2, const P& p3, const sphere& S)
+  void  back_dual(const P& p1, const P& p2, const P& p3, const Sphere& S)
   {
     const P ctr(0.3333*(p1+p2+p3));
 
@@ -257,52 +379,8 @@ namespace ePiX {
   }
 
 
-  // spherical plotting
-  void frontplot_N(double f1(double), double f2(double),
-		   double t_min, double t_max, int num_pts, 
-		   const sphere& S)
-  {
-    draw_sphereplot(f1, f2, zero, t_min, t_max, num_pts, true, STEREO_N, S);
-  }
-
-  void backplot_N(double f1(double), double f2(double),
-		  double t_min, double t_max, int num_pts,
-		  const sphere& S)
-  {
-    draw_sphereplot(f1, f2, zero, t_min, t_max, num_pts, false, STEREO_N, S);
-  }
-
-  void frontplot_S(double f1(double), double f2(double),
-		   double t_min, double t_max, int num_pts, 
-		   const sphere& S)
-  {
-    draw_sphereplot(f1, f2, zero, t_min, t_max, num_pts, true, STEREO_S, S);
-  }
-
-  void backplot_S(double f1(double), double f2(double),
-		  double t_min, double t_max, int num_pts, 
-		  const sphere& S)
-  {
-    draw_sphereplot(f1, f2, zero, t_min, t_max, num_pts, false, STEREO_S, S);
-  }
-
-
-  // Radial projection from center
-  void frontplot_R(P phi(double), double t_min, double t_max, 
-		   int num_pts, const sphere& S)
-  {
-    draw_sphereplot(phi, t_min, t_max, num_pts, true, RADIAL, S);
-  }
-
-  void backplot_R(P phi(double), double t_min, double t_max, 
-		  int num_pts, const sphere& S)
-  {
-    draw_sphereplot(phi, t_min, t_max, num_pts, false, RADIAL, S);
-  }
-
-
   // spherical polyhedra
-  void front_tetra(const sphere& S, const frame& coords)
+  void front_tetra(const Sphere& S, const frame& coords)
   {
     const P f1(coords.sea());
     const P f2(coords.sky());
@@ -318,7 +396,7 @@ namespace ePiX {
     front_triangle(ppp,nnp,pnn,S);
     front_triangle(nnp,pnn,npn,S);
   }
-  void  back_tetra(const sphere& S, const frame& coords)
+  void  back_tetra(const Sphere& S, const frame& coords)
   {
     const P f1(coords.sea());
     const P f2(coords.sky());
@@ -335,7 +413,7 @@ namespace ePiX {
     back_triangle(nnp,pnn,npn,S);
   }
 
-  void front_cube(const sphere& S, const frame& coords)
+  void front_cube(const Sphere& S, const frame& coords)
   {
     const P f1(coords.sea());
     const P f2(coords.sky());
@@ -366,7 +444,7 @@ namespace ePiX {
     front_arc(nnp,nnn,S);
     front_arc(pnp,pnn,S);
   }
-  void  back_cube(const sphere& S, const frame& coords)
+  void  back_cube(const Sphere& S, const frame& coords)
   {
     const P f1(coords.sea());
     const P f2(coords.sky());
@@ -398,7 +476,7 @@ namespace ePiX {
     back_arc(pnp,pnn,S);
   }
 
-  void front_octa(const sphere& S, const frame& coords)
+  void front_octa(const Sphere& S, const frame& coords)
   {
     const P p1(coords.sea());
     const P p2(coords.sky());
@@ -415,7 +493,7 @@ namespace ePiX {
     front_triangle(m1,p2,m3,S);
     front_triangle(p1,m2,m3,S);
   }
-  void  back_octa(const sphere& S, const frame& coords)
+  void  back_octa(const Sphere& S, const frame& coords)
   {
     const P p1(coords.sea());
     const P p2(coords.sky());
@@ -434,7 +512,7 @@ namespace ePiX {
   }
 
 
-  void front_dodeca(const sphere& S, const frame& coords)
+  void front_dodeca(const Sphere& S, const frame& coords)
   {
     const P f1(coords.sea());
     const P f2(coords.sky());
@@ -484,7 +562,7 @@ namespace ePiX {
     front_dual(mmo, omp, mop, S);
   }
 
-  void  back_dodeca(const sphere& S, const frame& coords)
+  void  back_dodeca(const Sphere& S, const frame& coords)
   {
     const P f1(coords.sea());
     const P f2(coords.sky());
@@ -535,7 +613,7 @@ namespace ePiX {
   }
 
 
-  void front_icosa(const sphere& S, const frame& coords)
+  void front_icosa(const Sphere& S, const frame& coords)
   {
     const P f1(coords.sea());
     const P f2(coords.sky());
@@ -585,7 +663,7 @@ namespace ePiX {
     front_triangle(mmo, omp, mop, S);
   }
 
-  void  back_icosa(const sphere& S, const frame& coords)
+  void  back_icosa(const Sphere& S, const frame& coords)
   {
     const P f1(coords.sea());
     const P f2(coords.sky());
@@ -636,6 +714,7 @@ namespace ePiX {
   }
 
 
+
   // Hyperbolic lines in upper half space
   // For compatibility with 2-D geometry, the boundary is the (x1,x3)-plane
 
@@ -711,16 +790,15 @@ namespace ePiX {
   void disk_line(const P& tail, const P& head)
   {
     const int N(EPIX_NUM_PTS);
-    std::vector<vertex> data(N+1);
+    std::vector<P> data(N+1);
 
     double t(0);
 
     for (int i=0; i <= N; ++i, t += 1.0/N)
-      data.at(i) = vertex(p_line(tail, head, t));
+      data.at(i) = p_line(tail, head, t);
 
-    path temp(data, false);
+    path temp(data, false, false);
     temp.draw();
   }
 
-} /* end of namespace */
-
+} // end of namespace
