@@ -4,12 +4,12 @@
  * This file is part of ePiX, a preprocessor for creating high-quality 
  * line figures in LaTeX 
  *
- * Version 1.0.20
- * Last Change: October 29, 2006
+ * Version 1.1.6
+ * Last Change: June 28, 2007
  */
 
 /* 
- * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006
+ * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007
  * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
  * Department of Mathematics and Computer Science
  * College of the Holy Cross
@@ -35,9 +35,50 @@
 #ifndef EPIX_GEOMETRY
 #define EPIX_GEOMETRY
 
+#include "frame.h"
+#include "sphere.h"
+
 namespace ePiX {
   class P;
-  class sphere;
+
+  // latitude, longitude lines with hiding
+  void latitude(double lat, double long_min, double long_max,
+		const Sphere& S=Sphere(), const frame& coords=frame());
+
+  void longitude(double lngtd, double lat_min, double lat_max,
+		 const Sphere& S=Sphere(), const frame& coords=frame());
+
+  void back_latitude(double lat, double long_min, double long_max,
+		     const Sphere& S=Sphere(), const frame& coords=frame());
+
+  void back_longitude(double lngtd, double lat_min, double lat_max,
+		      const Sphere& S=Sphere(), const frame& coords=frame());
+
+
+  // stereographic projection; path first projected vertically
+  void frontplot_N(double f1(double), double f2(double),
+		   double t_min, double t_max, int num_pts, 
+		   const Sphere& S = Sphere());
+
+  void backplot_N(double f1(double), double f2(double),
+		  double t_min, double t_max, int num_pts,
+		  const Sphere& S = Sphere());
+
+  void frontplot_S(double f1(double), double f2(double),
+		   double t_min, double t_max, int num_pts, 
+		   const Sphere& S = Sphere());
+
+  void backplot_S(double f1(double), double f2(double),
+		  double t_min, double t_max, int num_pts, 
+		  const Sphere& S = Sphere());
+
+  // Radial projection from center
+  void frontplot_R(P phi(double), double t_min, double t_max, 
+		   int num_pts, const Sphere& S = Sphere());
+
+  void backplot_R(P phi(double), double t_min, double t_max, 
+		  int num_pts, const Sphere& S = Sphere());
+
 
   // All sphere line functions treat point arguments as unscaled
   // displacements from the center of the sphere, not as absolute
@@ -45,59 +86,35 @@ namespace ePiX {
   // changes, the line automatically adjusts as expected.
 
   // front portion of spherical segment joining p1 to p2
-  void front_arc(const P& p1, const P& p2, const sphere& S = sphere());
-  void  back_arc(const P& p1, const P& p2, const sphere& S = sphere());
+  void front_arc(const P& p1, const P& p2, const Sphere& S = Sphere());
+  void  back_arc(const P& p1, const P& p2, const Sphere& S = Sphere());
 
   // half-line joining p1 to -p1 through p2
-  void front_arc2(const P&, const P&, const sphere& S=sphere());
-  void  back_arc2(const P&, const P&, const sphere& S=sphere());
+  void front_arc2(const P&, const P&, const Sphere& S=Sphere());
+  void  back_arc2(const P&, const P&, const Sphere& S=Sphere());
 
   // great circle through p1 and p2
-  void front_line(const P&, const P&, const sphere& S=sphere());
-  void  back_line(const P&, const P&, const sphere& S=sphere());
+  void front_line(const P&, const P&, const Sphere& S=Sphere());
+  void  back_line(const P&, const P&, const Sphere& S=Sphere());
 
-  void front_triangle(const P&, const P&, const P&, const sphere& S=sphere());
-  void  back_triangle(const P&, const P&, const P&, const sphere& S=sphere());
-
-  // stereographic projection; path first projected vertically
-  void frontplot_N(double f1(double), double f2(double),
-		   double t_min, double t_max, int num_pts, 
-		   const sphere& S = sphere());
-
-  void backplot_N(double f1(double), double f2(double),
-		  double t_min, double t_max, int num_pts,
-		  const sphere& S = sphere());
-
-  void frontplot_S(double f1(double), double f2(double),
-		   double t_min, double t_max, int num_pts, 
-		   const sphere& S = sphere());
-
-  void backplot_S(double f1(double), double f2(double),
-		  double t_min, double t_max, int num_pts, 
-		  const sphere& S = sphere());
-
-  // Radial projection from center
-  void frontplot_R(P phi(double), double t_min, double t_max, 
-		   int num_pts, const sphere& S = sphere());
-
-  void backplot_R(P phi(double), double t_min, double t_max, 
-		  int num_pts, const sphere& S = sphere());
+  void front_triangle(const P&, const P&, const P&, const Sphere& S=Sphere());
+  void  back_triangle(const P&, const P&, const P&, const Sphere& S=Sphere());
 
   // spherical polyhedra
-  void front_tetra(const sphere& S=sphere(), const frame& coords=frame());
-  void  back_tetra(const sphere& S=sphere(), const frame& coords=frame());
+  void front_tetra(const Sphere& S=Sphere(), const frame& coords=frame());
+  void  back_tetra(const Sphere& S=Sphere(), const frame& coords=frame());
 
-  void front_cube(const sphere& S=sphere(), const frame& coords=frame());
-  void  back_cube(const sphere& S=sphere(), const frame& coords=frame());
+  void front_cube(const Sphere& S=Sphere(), const frame& coords=frame());
+  void  back_cube(const Sphere& S=Sphere(), const frame& coords=frame());
 
-  void front_octa(const sphere& S=sphere(), const frame& coords=frame());
-  void  back_octa(const sphere& S=sphere(), const frame& coords=frame());
+  void front_octa(const Sphere& S=Sphere(), const frame& coords=frame());
+  void  back_octa(const Sphere& S=Sphere(), const frame& coords=frame());
 
-  void front_dodeca(const sphere& S=sphere(), const frame& coords=frame());
-  void  back_dodeca(const sphere& S=sphere(), const frame& coords=frame());
+  void front_dodeca(const Sphere& S=Sphere(), const frame& coords=frame());
+  void  back_dodeca(const Sphere& S=Sphere(), const frame& coords=frame());
 
-  void front_icosa(const sphere& S=sphere(), const frame& coords=frame());
-  void  back_icosa(const sphere& S=sphere(), const frame& coords=frame());
+  void front_icosa(const Sphere& S=Sphere(), const frame& coords=frame());
+  void  back_icosa(const Sphere& S=Sphere(), const frame& coords=frame());
 
   // Hyperbolic lines
   void hyperbolic_line (const P&, const P&);
