@@ -3,12 +3,12 @@
  *
  * Andrew D. Hwang  <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
  *
- * Version 1.1.5
- * Last Change: June 20, 2007
+ * Version 1.2.5
+ * Last Change: June 18, 2008
  */
 
 /* 
- * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007
+ * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
  * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
  * Department of Mathematics and Computer Science
  * College of the Holy Cross
@@ -59,10 +59,10 @@
 namespace ePiX {
 
   // identity and coordinate projection functions
-  template<class T>T id(T arg) { return arg; }
+  template<typename T>T id(T arg) { return arg; }
 
-  template<class T1, class T2>T1 proj1(T1 arg1, T2 arg2) { return arg1; }
-  template<class T1, class T2>T2 proj2(T1 arg1, T2 arg2) { return arg2; }
+  template<typename T1, typename T2>T1 proj1(T1 arg1, T2 arg2) { return arg1; }
+  template<typename T1, typename T2>T2 proj2(T1 arg1, T2 arg2) { return arg2; }
 
   // R -> R^3
   class path_map {
