@@ -4,12 +4,12 @@
  * This file is part of ePiX, a C++ library for creating high-quality 
  * figures in LaTeX 
  *
- * Version 1.1.3
- * Last Change: March 04, 2007
+ * Version 1.2.5
+ * Last Change: May 04, 2008
  */
 
 /* 
- * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007
+ * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
  * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
  * Department of Mathematics and Computer Science
  * College of the Holy Cross
@@ -115,6 +115,8 @@ namespace ePiX {
     // assumes path lies on sphere
     void clip_to(const Sphere&, const P& viewpt, bool back=false);
 
+    std::vector<P> data() const;
+
     void draw() const;
     void draw(const Color&, const pen_data&) const;
 
