@@ -1,11 +1,11 @@
 /* 
  * plots.cc: Plotting functions
  *
- * This file is part of ePiX, a preprocessor for creating high-quality 
- * line figures in LaTeX 
+ * This file is part of ePiX, a C++ library for creating high-quality 
+ * figures in LaTeX 
  *
- * Version 1.1
- * Last Change: January 01, 2007
+ * Version 1.2.0-2
+ * Last Change: September 26, 2007
  */
 
 /* 
@@ -32,70 +32,68 @@
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
-#include "globals.h"
+#include "constants.h"
 #include "errors.h"
 
 #include "triples.h"
 #include "functions.h"
 
-#include "lengths.h" // dart field must calculate page distances
-
+#include "camera.h"
 #include "map.h"
 #include "domain.h"
 
 #include "path.h"
-#include "cropping.h"
 
-#include "camera.h"
-#include "objects.h"
-#include "output.h"
+#include "markers.h"
 #include "curves.h"
+
+#include "deriv.h"
 #include "plot_algorithms.h"
 #include "plots.h"
 
 namespace ePiX {
 
-  extern epix_camera camera;
-
   typedef std::list<domain>::const_iterator dolci;
 
   // f:R -> R^3
-  void plot(P f(double), double t_min, double t_max, int num_pts)
+  void plot(P f(double), double t_min, double t_max, unsigned int num_pts)
   {
     plot_map_dom(path_map(f), domain(t_min, t_max, num_pts));
   }
 
   // f:R -> R x R x R
   void plot(double f1(double), double f2(double), double f3(double),
-	    double t_min, double t_max, int num_pts)
+	    double t_min, double t_max, unsigned int num_pts)
   {
     plot_map_dom(column_1var(f1,f2,f3), domain(t_min, t_max, num_pts));
   }
 
   // f:R -> R
-  void plot(double f(double), double t_min, double t_max, int num_pts)
+  void plot(double f(double), double t_min, double t_max, unsigned int num_pts)
   {
     plot(id, f, zero, t_min, t_max, num_pts); 
   }
 
-  void plot(const Deriv& f, double t_min, double t_max, int num_pts)
+  void plot(const Deriv& f, double t_min, double t_max, unsigned int num_pts)
   { 
     plot_map_dom(f, domain(t_min, t_max, num_pts));
   }
 
-  void plot(const Integral& f, double t_min, double t_max, int num_pts)
+  void plot(const Integral& f, double t_min, double t_max,
+	    unsigned int num_pts)
   {
     plot_map_dom(f, domain(t_min, t_max, num_pts));
   }
 
   // f: R -> R x R
   void plot(double f1(double), double f2(double), 
-	    double t_min, double t_max, int num_pts)
+	    double t_min, double t_max, unsigned int num_pts)
   {
     plot(f1, f2, zero, t_min, t_max, num_pts); 
   }
 
-  void polarplot (double f(double), double t_min, double t_max, int num_pts)
+  void polarplot (double f(double), double t_min, double t_max,
+		  unsigned int num_pts)
   {
     plot_map_dom(column_1var(f, id, zero, cylindrical), 
 		 domain(t_min, t_max, num_pts));
@@ -157,19 +155,22 @@ namespace ePiX {
 
 
   // Derivatives and integrals
-  void plot_deriv(double f(double t), double a, double b, int num_pts)
+  void plot_deriv(double f(double t), double a, double b,
+		  unsigned int num_pts)
   {
-    plot_function(D(f), a, b, num_pts);
+    plot_function(Deriv(f), a, b, num_pts);
   }
 
-  void plot_int(double f(double), double a, double b, int num_pts)
+  void plot_int(double f(double), double a, double b,
+		unsigned int num_pts)
   {
-    plot_function(I(f, a), a, b, num_pts);
+    plot_function(Integral(f, a), a, b, num_pts);
   }
 
-  void plot_int(double f(double), double x0, double a, double b, int num_pts)
+  void plot_int(double f(double), double x0, double a, double b,
+		unsigned int num_pts)
   {
-    plot_function(I(f, x0), a, b, num_pts);
+    plot_function(Integral(f, x0), a, b, num_pts);
   }
 
   // tuples of real-valued plot arguments
@@ -185,22 +186,23 @@ namespace ePiX {
   }
 
   void envelope(double f1(double t), double f2(double t), 
-		double t_min, double t_max, int num_pts)
+		double t_min, double t_max, unsigned int num_pts)
   {
     const double step((t_max - t_min)/num_pts);
     double t(t_min);
 
-    for (int i=0; i <= num_pts; ++i, t += step)
+    for (unsigned int i=0; i <= num_pts; ++i, t += step)
       Line(P(f1(t), f2(t)),
 	   P(f1(t), f2(t)) + P(deriv(f1, t), deriv(f2, t)));
   }	      
 
-  void envelope(double f(double t), double t_min, double t_max, int num_pts)
+  void envelope(double f(double t), double t_min, double t_max,
+		unsigned int num_pts)
   {
     const double step((t_max - t_min)/num_pts);
     double t(t_min);
 
-    for (int i=0; i <= num_pts; ++i, t += step)
+    for (unsigned int i=0; i <= num_pts; ++i, t += step)
       Line(P(t, f(t)), P(t, f(t)) + P(1, deriv(f, t)));
   }	      
 
@@ -210,39 +212,37 @@ namespace ePiX {
     Line(f(t0), f(t0) + deriv(f, t0));
   }
 
-  void envelope(P f(double t), double t_min, double t_max, int num_pts)
+  void envelope(P f(double t), double t_min, double t_max, 
+		unsigned int num_pts)
   {
     const double step((t_max - t_min)/num_pts);
     double t(t_min);
 
-    for (int i=0; i <= num_pts; ++i, t += step)
+    for (unsigned int i=0; i <= num_pts; ++i, t += step)
       Line(f(t), f(t) + deriv(f, t));
   }	      
 
   // Tangent field along parametrized path
   void tan_field(double f1(double), double f2(double), double t_min,
-		 double t_max, double num_pts)
+		 double t_max, unsigned int num_pts)
   {
     const double step((t_max - t_min)/num_pts);
     double t(t_min);
 
-    for (int i=0; i <= num_pts; ++i, t += step)
+    for (unsigned int i=0; i <= num_pts; ++i, t += step)
       arrow(P(f1(t), f2(t)), P(f1(t), f2(t)) +
 	    step*P(deriv(f1, t), deriv(f2, t)));
-
-    end_stanza();
   }
 
   // P-valued plot argument
-  void tan_field(P f(double), double t_min, double t_max, double num_pts)
+  void tan_field(P f(double), double t_min, double t_max,
+		 unsigned int num_pts)
   {
     const double step((t_max - t_min)/num_pts);
     double t(t_min);
 
-    for (int i=0; i <= num_pts; ++i, t += step)
+    for (unsigned int i=0; i <= num_pts; ++i, t += step)
       arrow(f(t), f(t) + step*deriv(f, t));
-
-    end_stanza();
   }
 
   // Slope, dart, and vector fields
@@ -250,7 +250,7 @@ namespace ePiX {
   // utility functions
   // vector, dart, and slope fields
   void field_element(const P& tail, const P& head, epix_field_type TYPE,
-		     const double head_scale=1.0)
+		     double head_scale=1.0)
   {
     switch(TYPE) {
 
@@ -271,8 +271,10 @@ namespace ePiX {
     } // end of switch(TYPE)
   } // end of field_element
 
-  void draw_field(P F(double, double, double), P p, P q, int n1, int n2, 
-		  epix_field_type TYPE, const double scale)
+
+  void draw_field(P F(double, double, double), P p, P q,
+		  unsigned int n1, unsigned int n2, 
+		  epix_field_type TYPE, double scale)
   {
     P diagonal(q-p);
     int perp_count(0);
@@ -314,8 +316,8 @@ namespace ePiX {
     const double d1(norm(jump1));
     const double d2(norm(jump2));
 
-    for (int i=0; i <= n1; ++i)
-      for (int j=0; j <= n2; ++j)
+    for (unsigned int i=0; i <= n1; ++i)
+      for (unsigned int j=0; j <= n2; ++j)
 	{
 	  P base(p + (i*jump1 + j*jump2));
 	  P vect(F(base.x1(), base.x2(), base.x3()));
@@ -323,16 +325,18 @@ namespace ePiX {
 	  if (norm(camera(base+vect) - camera(base)) < EPIX_EPSILON)
 	    marker(base, BBOX);
 
-	  // scale affects head size
 	  else if (TYPE == VECTOR)
 	    field_element(base, base + vect, VECTOR, scale);
 
 	  else
 	    {
-	      double object_len(norm(c2s(camera(base+vect)-camera(base))));
-	      pair dX(c2s(pair(d1, d2)));
+	      // N.B. May not work if page export bolloxes aspect ratio
+	      double object_len(norm(camera(base+vect)-camera(base)));
+	      pair dX(pair(d1, d2));
+
 	      // Hardwired constant 0.4: Segment 80% of shorter grid length
 	      double minimum((dX.x1() > dX.x2()) ? 0.4*dX.x2() : 0.4*dX.x1());
+
 	      if ( object_len > EPIX_EPSILON) // not projected to zero
 		vect *= minimum/object_len;
 
@@ -340,12 +344,12 @@ namespace ePiX {
 	      field_element(base - scale*vect, base + scale*vect, TYPE);
 	    }
 	}
-    end_stanza();
   } // end of draw_field F(x, y, z)
 
 
-  void draw_field(P F(double, double), P p, P q, int n1, int n2, 
-		  epix_field_type TYPE, const double scale)
+  void draw_field(P F(double, double), P p, P q,
+		  unsigned int n1, unsigned int n2, 
+		  epix_field_type TYPE, double scale)
   {
     P diagonal(q-p);
     P jump1(E_1&diagonal);
@@ -360,8 +364,8 @@ namespace ePiX {
     const double d1(norm(jump1));
     const double d2(norm(jump2));
 
-    for (int i=0; i <= n1; ++i)
-      for (int j=0; j <= n2; ++j)
+    for (unsigned int i=0; i <= n1; ++i)
+      for (unsigned int j=0; j <= n2; ++j)
 	{
 	  P base(p + (i*jump1 + j*jump2));
 	  P vect(F(base.x1(), base.x2()));
@@ -375,8 +379,9 @@ namespace ePiX {
 
 	  else
 	    {
-	      double object_len(norm(c2s(camera(base+vect)-camera(base))));
-	      pair dX(c2s(pair(d1, d2)));
+	      // see caution above
+	      double object_len(norm(camera(base+vect)-camera(base)));
+	      pair dX(pair(d1, d2));
 	      // Hardwired constant 0.4: Segment 80% of shorter grid length
 	      double minimum((dX.x1() > dX.x2()) ? 0.4*dX.x2() : 0.4*dX.x1());
 	      if ( object_len > EPIX_EPSILON) // not projected to zero
@@ -386,155 +391,157 @@ namespace ePiX {
 	      field_element(base - scale*vect, base + scale*vect, TYPE);
 	    }
 	}
-    end_stanza();
   } // end of draw_field F(x, y)
 
 
   // planar fields
-  void slope_field(P F(double, double),
-		   const P& p, const P& q, int n1, int n2, const double scale)
+  void slope_field(P F(double, double), const P& p, const P& q,
+		   unsigned int n1, unsigned int n2, double scale)
   {
     draw_field(F, p, q, n1, n2, SLOPE, scale);
   }
 
-  void dart_field(P F(double, double),
-		  const P& p, const P& q, int n1, int n2, const double scale)
+  void dart_field(P F(double, double), const P& p, const P& q,
+		  unsigned int n1, unsigned int n2, double scale)
   {
     draw_field(F, p, q, n1, n2, DART, scale);
   }
 
-  void vector_field(P F(double, double),
-		    const P& p, const P& q, int n1, int n2, const double scale)
+  void vector_field(P F(double, double), const P& p, const P& q,
+		    unsigned int n1, unsigned int n2, double scale)
   {
     draw_field(F, p, q, n1, n2, VECTOR, scale);
   }
 
-  void  slope_field(P F(double, double), const domain& R, const double scale)
+  void  slope_field(P F(double, double), const domain& R, double scale)
   {
-    draw_field(F, R.corner1, R.corner2,
-	       R.coarse.n1(), R.coarse.n2(),
+    draw_field(F, R.corner1(), R.corner2(),
+	       R.coarse_n1(), R.coarse_n2(),
 	       SLOPE, scale);
   }
 
-  void   dart_field(P F(double, double), const domain& R, const double scale)
+  void   dart_field(P F(double, double), const domain& R, double scale)
   {
-    draw_field(F, R.corner1, R.corner2,
-	       R.coarse.n1(), R.coarse.n2(), 
+    draw_field(F, R.corner1(), R.corner2(),
+	       R.coarse_n1(), R.coarse_n2(), 
 	       DART, scale);
   }
 
-  void vector_field(P F(double, double), const domain& R, const double scale)
+  void vector_field(P F(double, double), const domain& R, double scale)
   {
-    draw_field(F, R.corner1, R.corner2,
-	       R.coarse.n1(), R.coarse.n2(), 
+    draw_field(F, R.corner1(), R.corner2(),
+	       R.coarse_n1(), R.coarse_n2(), 
 	       VECTOR, scale);
   }
 
   // spatial fields
-  void  slope_field(P F(double, double, double),
-		    const P& p, const P& q, int n1, int n2, const double scale)
+  void  slope_field(P F(double, double, double), const P& p, const P& q,
+		    unsigned int n1, unsigned int n2, double scale)
   {
     draw_field(F, p, q, n1, n2, SLOPE, scale);
   }
 
-  void   dart_field(P F(double, double, double),
-		    const P& p, const P& q, int n1, int n2, const double scale)
+  void   dart_field(P F(double, double, double), const P& p, const P& q,
+		    unsigned int n1, unsigned int n2, double scale)
   {
     draw_field(F, p, q, n1, n2, DART, scale);
   }
 
-  void vector_field(P F(double, double, double), 
-		    const P& p, const P& q, int n1, int n2, const double scale)
+  void vector_field(P F(double, double, double), const P& p, const P& q,
+		    unsigned int n1, unsigned int n2, double scale)
   {
     draw_field(F, p, q, n1, n2, VECTOR, scale);
   }
 
   // spatial fields over a domain
   void  slope_field(P F(double, double, double), const domain& R,
-		    const double scale)
+		    double scale)
   {
-    P p(R.corner1), q(R.corner2);
+    P p(R.corner1()), q(R.corner2());
     double height(q.x3() - p.x3());
     q -= height*E_3; // p.x3() == q.x3()
 
-    int i_max(R.coarse.n1()), j_max(R.coarse.n2()); // horizontal subdivisions
-    int k_max((R.dx3() > 0) ? R.coarse.n3() : 0); // vertical subdivisions
-
-    for (int k = 0; k <= k_max; ++k)
+    // horizontal/vertical subdivisions
+    unsigned int i_max(R.coarse_n1()), j_max(R.coarse_n2());
+    unsigned int k_max((R.dx3() > 0) ? R.coarse_n3() : 0);
+    for (unsigned int k = 0; k <= k_max; ++k)
       draw_field(F, p + (k*R.step3())*E_3, q + (k*R.step3())*E_3,
 		 i_max, j_max, SLOPE, scale);
   }
 
   void   dart_field(P F(double, double, double), const domain& R,
-		    const double scale)
+		    double scale)
   {
-    P p(R.corner1), q(R.corner2);
+    P p(R.corner1()), q(R.corner2());
     double height(q.x3() - p.x3());
     q -= height*E_3; // p.x3() == q.x3()
 
-    int i_max(R.coarse.n1()), j_max(R.coarse.n2()); // horizontal subdivisions
-    int k_max((R.dx3() > 0) ? R.coarse.n3() : 0); // vertical subdivisions
+    unsigned int i_max(R.coarse_n1()), j_max(R.coarse_n2());
+    unsigned int k_max((R.dx3() > 0) ? R.coarse_n3() : 0);
 
-    for (int k = 0; k <= k_max; ++k)
+    for (unsigned int k = 0; k <= k_max; ++k)
       draw_field(F, p + (k*R.step3())*E_3, q + (k*R.step3())*E_3,
 		 i_max, j_max, DART, scale);
   }
 
   void vector_field(P F(double, double, double), const domain& R,
-		    const double scale)
+		    double scale)
   {
-    P p(R.corner1), q(R.corner2);
+    P p(R.corner1()), q(R.corner2());
     double height(q.x3() - p.x3());
     q -= height*E_3; // p.x3() == q.x3()
 
-    int i_max(R.coarse.n1()), j_max(R.coarse.n2()); // horizontal subdivisions
-    int k_max((R.dx3() > 0) ? R.coarse.n3() : 0); // vertical subdivisions
+    unsigned int i_max(R.coarse_n1()), j_max(R.coarse_n2());
+    unsigned int k_max((R.dx3() > 0) ? R.coarse_n3() : 0);
 
-    for (int k = 0; k <= k_max; ++k)
+    for (unsigned int k = 0; k <= k_max; ++k)
       draw_field(F, p + (k*R.step3())*E_3, q + (k*R.step3())*E_3,
 		 i_max, j_max, VECTOR, scale);
   }
 
   // Solutions of ODE systems
   // start at time 0
-  void ode_plot (P F(double, double), const P& start, double t_max, int n)
+  void ode_plot (P F(double, double), const P& start,
+		 double t_max, unsigned int n)
   {
     euler_plot(surface_map(F), start, 0, t_max, n);
   }
 
   void ode_plot (P F(double, double, double), const P& start,
-		 double t_max, int num_pts)
+		 double t_max, unsigned int num_pts)
   {
     euler_plot(space_map(F), start, 0, t_max, num_pts);
   }
 
   // arbitrary start time
   void ode_plot (P F(double, double), const P& start, 
-		 double t_min, double t_max, int num_pts)
+		 double t_min, double t_max, unsigned int num_pts)
   {
     euler_plot(surface_map(F), start, t_min, t_max, num_pts);
   }
 
   void ode_plot (P F(double, double, double), const P& start, 
-		 double t_min, double t_max, int num_pts)
+		 double t_min, double t_max, unsigned int num_pts)
   {
     euler_plot(space_map(F), start, t_min, t_max, num_pts);
   }
 
 
   // flow x0 under field for specified time
-  P flow (P F(double, double), const P& start, double t_max, int n)
+  P flow (P F(double, double), const P& start, double t_max, 
+	  unsigned int n)
   {
     return euler_flow(surface_map(F), start, t_max, n);
   }
 
-  P flow (P F(double, double, double), const P& start, double t_max, int n)
+  P flow (P F(double, double, double), const P& start, double t_max,
+	  unsigned int n)
   {
     return euler_flow(space_map(F), start, t_max, n);
   }
 
 
-  void riemann_sum(double f(double), const double a, const double b,
+  void riemann_sum(double f(double), double a, double b,
 		   unsigned int n, epix_integral_type TYPE)
   {
     const double dx((b-a)/n);
@@ -579,71 +586,28 @@ namespace ePiX {
 
   // Jay Belanger's shaded plotting functions -- December 1, 2002
   // Re-implemented using paths, July 16, 2004
+  //        "       for Version 1.2, June 2007
   void shadeplot(double f1(double), double f2(double), 
-		 double t_min, double t_max, int num_pts)
+		 double t_min, double t_max, unsigned int num_pts)
   {
     // build contour
     path boundary(f1, t_min, t_max, num_pts);
     path graph2(f2, t_min, t_max, num_pts);
 
-    boundary -= graph2;
+    path right_edge(P(t_max, f1(t_max)), P(t_max, f2(t_max)));
+    path  left_edge(P(t_min, f2(t_min)), P(t_min, f1(t_min)));
 
-    boundary.close();
-    boundary.set_fill();
+    boundary += right_edge;
+    boundary -= graph2;
+    boundary += left_edge;
+
+    boundary.close().fill();
     boundary.draw();
-    end_stanza();
   } // end of shadeplot
 
-  void shadeplot(double f(double), double t_min, double t_max, int num_pts)
+  void shadeplot(double f(double), double t_min, double t_max,
+		 unsigned int num_pts)
   {
     shadeplot(f, zero, t_min, t_max, num_pts);
   }
-
-  void blackplot(double f1(double), double f2(double), 
-		 double t_min, double t_max, int num_pts)
-  {
-    double temp_gray(epix::get_gray()); // get state
-    gray(1);
-    shadeplot(f1, f2, t_min, t_max, num_pts);
-    gray(temp_gray); // restore
-  }
-
-  void blackplot(double f(double), double t_min, double t_max, int num_pts)
-  {
-    blackplot(f, zero, t_min, t_max, num_pts);
-  }
-
-  void whiteplot(double f1(double), double f2(double), 
-		 double t_min, double t_max, int num_pts)
-  {
-    double temp_gray(epix::get_gray());
-    gray(0);
-    shadeplot(f1, f2, t_min, t_max, num_pts);
-    gray(temp_gray);
-  }
-
-  void whiteplot(double f(double), double t_min, double t_max, int num_pts)
-  {
-    whiteplot(f, zero, t_min, t_max, num_pts);
-  }
-
-
-  // wiremesh surface plotting
-  void clipplot(P Phi(double, double),
-		const P& p1, const P& p2, const mesh& N, const mesh& num_pts)
-  {
-    bool temp_clip(epix::clipping); // get state
-    clip();
-    plot_map_dom(surface_map(Phi), domain(p1, p2, N, num_pts));
-    clip(temp_clip); // restore
-  }
-
-  void cropplot(P Phi(double, double),
-	    const P& p1, const P& p2, const mesh& N, const mesh& num_pts)
-  {
-    bool temp_crop(epix::cropping);
-    crop();
-    plot_map_dom(surface_map(Phi), domain(p1, p2, N, num_pts));
-    crop(temp_crop);
-  }
 } // end of namespace
